@@ -1,5 +1,5 @@
 /* GSequencer - Advanced GTK Sequencer
- * Copyright (C) 2005-2018 Joël Krähemann
+ * Copyright (C) 2005-2019 Joël Krähemann
  *
  * This file is part of GSequencer.
  *
@@ -19,8 +19,6 @@
 
 #include <ags/audio/ags_buffer.h>
 
-#include <ags/libags.h>
-
 #include <ags/audio/ags_audio_signal.h>
 #include <ags/audio/ags_audio_buffer_util.h>
 
@@ -29,8 +27,6 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include <pthread.h>
-
 void ags_buffer_class_init(AgsBufferClass *buffer);
 void ags_buffer_init(AgsBuffer *buffer);
 void ags_buffer_set_property(GObject *gobject,
@@ -66,8 +62,6 @@ enum{
 
 static gpointer ags_buffer_parent_class = NULL;
 
-static pthread_mutex_t ags_buffer_class_mutex = PTHREAD_MUTEX_INITIALIZER;
-
 GType
 ags_buffer_get_type()
 {
@@ -121,7 +115,7 @@ ags_buffer_class_init(AgsBufferClass *buffer)
    *
    * Buffer's x offset.
    * 
-   * Since: 2.0.0
+   * Since: 3.0.0
    */
   param_spec = g_param_spec_uint64("x",
 				   i18n_pspec("offset x"),
@@ -139,7 +133,7 @@ ags_buffer_class_init(AgsBufferClass *buffer)
    *
    * Buffer's selection x0 offset.
    * 
-   * Since: 2.0.0
+   * Since: 3.0.0
    */
   param_spec = g_param_spec_uint64("selection-x0",
 				   i18n_pspec("selection offset x0"),
@@ -157,7 +151,7 @@ ags_buffer_class_init(AgsBufferClass *buffer)
    *
    * Buffer's selection x1 offset.
    * 
-   * Since: 2.0.0
+   * Since: 3.0.0
    */
   param_spec = g_param_spec_uint64("selection-x1",
 				   i18n_pspec("selection offset x1"),
@@ -175,7 +169,7 @@ ags_buffer_class_init(AgsBufferClass *buffer)
    *
    * Buffer's audio data samplerate.
    * 
-   * Since: 2.0.0
+   * Since: 3.0.0
    */
   param_spec = g_param_spec_uint("samplerate",
 				 i18n_pspec("audio data samplerate"),
@@ -193,7 +187,7 @@ ags_buffer_class_init(AgsBufferClass *buffer)
    *
    * Buffer's audio data buffer size.
    * 
-   * Since: 2.0.0
+   * Since: 3.0.0
    */
   param_spec = g_param_spec_uint("buffer-size",
 				 i18n_pspec("audio data's buffer size"),
@@ -211,7 +205,7 @@ ags_buffer_class_init(AgsBufferClass *buffer)
    *
    * Buffer's audio data format.
    * 
-   * Since: 2.0.0
+   * Since: 3.0.0
    */
   param_spec = g_param_spec_uint("format",
 				 i18n_pspec("audio data format"),
@@ -229,7 +223,7 @@ ags_buffer_class_init(AgsBufferClass *buffer)
    *
    * Buffer's audio data.
    * 
-   * Since: 2.0.0
+   * Since: 3.0.0
    */
   param_spec = g_param_spec_pointer("data",
 				    i18n_pspec("audio data"),
@@ -245,28 +239,11 @@ ags_buffer_init(AgsBuffer *buffer)
 {  
   AgsConfig *config;
 
-  pthread_mutex_t *mutex;
-  pthread_mutexattr_t *attr;
-  
   buffer->flags = 0;
 
   /* add buffer mutex */
-  buffer->obj_mutexattr = 
-    attr = (pthread_mutexattr_t *) malloc(sizeof(pthread_mutexattr_t));
-  pthread_mutexattr_init(attr);
-  pthread_mutexattr_settype(attr,
-			    PTHREAD_MUTEX_RECURSIVE);
-
-#ifdef __linux__
-  pthread_mutexattr_setprotocol(attr,
-				PTHREAD_PRIO_INHERIT);
-#endif
-
-  buffer->obj_mutex = 
-    mutex = (pthread_mutex_t *) malloc(sizeof(pthread_mutex_t));
-  pthread_mutex_init(mutex,
-		     attr);  
-
+  g_rec_mutex_init(&(buffer->obj_mutex));
+  
   /* config */
   config = ags_config_get_instance();
 
@@ -292,43 +269,39 @@ ags_buffer_set_property(GObject *gobject,
 {
   AgsBuffer *buffer;
 
-  pthread_mutex_t *buffer_mutex;
+  GRecMutex *buffer_mutex;
 
   buffer = AGS_BUFFER(gobject);
 
   /* get buffer mutex */
-  pthread_mutex_lock(ags_buffer_get_class_mutex());
-  
-  buffer_mutex = buffer->obj_mutex;
-  
-  pthread_mutex_unlock(ags_buffer_get_class_mutex());
+  buffer_mutex = AGS_BUFFER_GET_OBJ_MUTEX(buffer);
 
   switch(prop_id){
   case PROP_X:
     {
-      pthread_mutex_lock(buffer_mutex);
+      g_rec_mutex_lock(buffer_mutex);
 
       buffer->x = g_value_get_uint64(value);
 
-      pthread_mutex_unlock(buffer_mutex);
+      g_rec_mutex_unlock(buffer_mutex);
     }
     break;
   case PROP_SELECTION_X0:
     {
-      pthread_mutex_lock(buffer_mutex);
+      g_rec_mutex_lock(buffer_mutex);
 
       buffer->selection_x0 = g_value_get_uint64(value);
 
-      pthread_mutex_unlock(buffer_mutex);
+      g_rec_mutex_unlock(buffer_mutex);
     }
     break;
   case PROP_SELECTION_X1:
     {
-      pthread_mutex_lock(buffer_mutex);
+      g_rec_mutex_lock(buffer_mutex);
 
       buffer->selection_x1 = g_value_get_uint64(value);
 
-      pthread_mutex_unlock(buffer_mutex);
+      g_rec_mutex_unlock(buffer_mutex);
     }
     break;
   case PROP_SAMPLERATE:
@@ -363,11 +336,11 @@ ags_buffer_set_property(GObject *gobject,
     break;
   case PROP_DATA:
     {
-      pthread_mutex_lock(buffer_mutex);
+      g_rec_mutex_lock(buffer_mutex);
 
       buffer->data = g_value_get_pointer(value);
 
-      pthread_mutex_unlock(buffer_mutex);
+      g_rec_mutex_unlock(buffer_mutex);
     }
     break;
   default:
@@ -384,86 +357,82 @@ ags_buffer_get_property(GObject *gobject,
 {
   AgsBuffer *buffer;
 
-  pthread_mutex_t *buffer_mutex;
+  GRecMutex *buffer_mutex;
 
   buffer = AGS_BUFFER(gobject);
 
   /* get buffer mutex */
-  pthread_mutex_lock(ags_buffer_get_class_mutex());
-  
-  buffer_mutex = buffer->obj_mutex;
-  
-  pthread_mutex_unlock(ags_buffer_get_class_mutex());
+  buffer_mutex = AGS_BUFFER_GET_OBJ_MUTEX(buffer);
 
   switch(prop_id){
   case PROP_X:
     {
-      pthread_mutex_lock(buffer_mutex);
+      g_rec_mutex_lock(buffer_mutex);
 
       g_value_set_uint64(value,
 			 buffer->x);
 
-      pthread_mutex_unlock(buffer_mutex);
+      g_rec_mutex_unlock(buffer_mutex);
     }
     break;
   case PROP_SELECTION_X0:
     {
-      pthread_mutex_lock(buffer_mutex);
+      g_rec_mutex_lock(buffer_mutex);
 
       g_value_set_uint64(value,
 			 buffer->selection_x0);
 
-      pthread_mutex_unlock(buffer_mutex);
+      g_rec_mutex_unlock(buffer_mutex);
     }
     break;
   case PROP_SELECTION_X1:
     {
-      pthread_mutex_lock(buffer_mutex);
+      g_rec_mutex_lock(buffer_mutex);
 
       g_value_set_uint64(value,
 			 buffer->selection_x1);
 
-      pthread_mutex_unlock(buffer_mutex);
+      g_rec_mutex_unlock(buffer_mutex);
     }
     break;
   case PROP_SAMPLERATE:
     {
-      pthread_mutex_lock(buffer_mutex);
+      g_rec_mutex_lock(buffer_mutex);
 
       g_value_set_uint(value,
 		       buffer->samplerate);
 
-      pthread_mutex_unlock(buffer_mutex);
+      g_rec_mutex_unlock(buffer_mutex);
     }
     break;
   case PROP_BUFFER_SIZE:
     {
-      pthread_mutex_lock(buffer_mutex);
+      g_rec_mutex_lock(buffer_mutex);
 
       g_value_set_uint(value,
 		       buffer->buffer_size);
 
-      pthread_mutex_unlock(buffer_mutex);
+      g_rec_mutex_unlock(buffer_mutex);
     }
     break;
   case PROP_FORMAT:
     {
-      pthread_mutex_lock(buffer_mutex);
+      g_rec_mutex_lock(buffer_mutex);
 
       g_value_set_uint(value,
 		       buffer->format);
 
-      pthread_mutex_unlock(buffer_mutex);
+      g_rec_mutex_unlock(buffer_mutex);
     }
     break;
   case PROP_DATA:
     {
-      pthread_mutex_lock(buffer_mutex);
+      g_rec_mutex_lock(buffer_mutex);
 
       g_value_set_pointer(value,
 			  buffer->data);
 
-      pthread_mutex_unlock(buffer_mutex);
+      g_rec_mutex_unlock(buffer_mutex);
     }
     break;
   default:
@@ -479,12 +448,6 @@ ags_buffer_finalize(GObject *gobject)
 
   buffer = AGS_BUFFER(gobject);
 
-  pthread_mutex_destroy(buffer->obj_mutex);
-  free(buffer->obj_mutex);
-
-  pthread_mutexattr_destroy(buffer->obj_mutexattr);
-  free(buffer->obj_mutexattr);
-
   if(buffer->data != NULL){
     free(buffer->data);
   }
@@ -493,21 +456,6 @@ ags_buffer_finalize(GObject *gobject)
   G_OBJECT_CLASS(ags_buffer_parent_class)->finalize(gobject);
 }
 
-/**
- * ags_buffer_get_class_mutex:
- * 
- * Use this function's returned mutex to access mutex fields.
- *
- * Returns: the class mutex
- * 
- * Since: 2.0.0
- */
-pthread_mutex_t*
-ags_buffer_get_class_mutex()
-{
-  return(&ags_buffer_class_mutex);
-}
-
 /**
  * ags_buffer_test_flags:
  * @buffer: the #AgsBuffer
@@ -517,32 +465,28 @@ ags_buffer_get_class_mutex()
  * 
  * Returns: %TRUE if flags are set, else %FALSE
  * 
- * Since: 2.0.0
+ * Since: 3.0.0
  */
 gboolean
 ags_buffer_test_flags(AgsBuffer *buffer, guint flags)
 {
   gboolean retval;
   
-  pthread_mutex_t *buffer_mutex;
+  GRecMutex *buffer_mutex;
 
   if(!AGS_IS_BUFFER(buffer)){
     return(FALSE);
   }
       
-  /* get buffer mutex */
-  pthread_mutex_lock(ags_buffer_get_class_mutex());
-  
-  buffer_mutex = buffer->obj_mutex;
-  
-  pthread_mutex_unlock(ags_buffer_get_class_mutex());
+  /* get buffer mutex */  
+  buffer_mutex = AGS_BUFFER_GET_OBJ_MUTEX(buffer);
 
   /* test */
-  pthread_mutex_lock(buffer_mutex);
+  g_rec_mutex_lock(buffer_mutex);
 
   retval = (flags & (buffer->flags)) ? TRUE: FALSE;
   
-  pthread_mutex_unlock(buffer_mutex);
+  g_rec_mutex_unlock(buffer_mutex);
 
   return(retval);
 }
@@ -554,30 +498,26 @@ ags_buffer_test_flags(AgsBuffer *buffer, guint flags)
  * 
  * Set @flags on @buffer.
  * 
- * Since: 2.0.0
+ * Since: 3.0.0
  */
 void
 ags_buffer_set_flags(AgsBuffer *buffer, guint flags)
 {
-  pthread_mutex_t *buffer_mutex;
+  GRecMutex *buffer_mutex;
 
   if(!AGS_IS_BUFFER(buffer)){
     return;
   }
       
   /* get buffer mutex */
-  pthread_mutex_lock(ags_buffer_get_class_mutex());
-  
-  buffer_mutex = buffer->obj_mutex;
-  
-  pthread_mutex_unlock(ags_buffer_get_class_mutex());
+  buffer_mutex = AGS_BUFFER_GET_OBJ_MUTEX(buffer);
 
   /* set */
-  pthread_mutex_lock(buffer_mutex);
+  g_rec_mutex_lock(buffer_mutex);
 
   buffer->flags |= flags;
   
-  pthread_mutex_unlock(buffer_mutex);
+  g_rec_mutex_unlock(buffer_mutex);
 }
 
 /**
@@ -587,30 +527,26 @@ ags_buffer_set_flags(AgsBuffer *buffer, guint flags)
  * 
  * Unset @flags on @buffer.
  * 
- * Since: 2.0.0
+ * Since: 3.0.0
  */
 void
 ags_buffer_unset_flags(AgsBuffer *buffer, guint flags)
 {
-  pthread_mutex_t *buffer_mutex;
+  GRecMutex *buffer_mutex;
 
   if(!AGS_IS_BUFFER(buffer)){
     return;
   }
       
   /* get buffer mutex */
-  pthread_mutex_lock(ags_buffer_get_class_mutex());
-  
-  buffer_mutex = buffer->obj_mutex;
-  
-  pthread_mutex_unlock(ags_buffer_get_class_mutex());
+  buffer_mutex = AGS_BUFFER_GET_OBJ_MUTEX(buffer);
 
   /* unset */
-  pthread_mutex_lock(buffer_mutex);
+  g_rec_mutex_lock(buffer_mutex);
 
   buffer->flags &= (~flags);
   
-  pthread_mutex_unlock(buffer_mutex);
+  g_rec_mutex_unlock(buffer_mutex);
 }
 
 /**
@@ -620,32 +556,28 @@ ags_buffer_unset_flags(AgsBuffer *buffer, guint flags)
  * 
  * Set samplerate.
  * 
- * Since: 2.0.0
+ * Since: 3.0.0
  */
 void
 ags_buffer_set_samplerate(AgsBuffer *buffer,
 			  guint samplerate)
 {
-  pthread_mutex_t *buffer_mutex;
+  GRecMutex *buffer_mutex;
 
   if(!AGS_IS_BUFFER(buffer)){
     return;
   }
 
   /* get buffer mutex */
-  pthread_mutex_lock(ags_buffer_get_class_mutex());
-  
-  buffer_mutex = buffer->obj_mutex;
-  
-  pthread_mutex_unlock(ags_buffer_get_class_mutex());
+  buffer_mutex = AGS_BUFFER_GET_OBJ_MUTEX(buffer);
 
   /* set samplerate */
-  pthread_mutex_lock(buffer_mutex);
+  g_rec_mutex_lock(buffer_mutex);
 
   //TODO:JK: resample data?
   buffer->samplerate = samplerate;
   
-  pthread_mutex_unlock(buffer_mutex);
+  g_rec_mutex_unlock(buffer_mutex);
 }
 
 /**
@@ -655,7 +587,7 @@ ags_buffer_set_samplerate(AgsBuffer *buffer,
  * 
  * Set buffer size.
  * 
- * Since: 2.0.0
+ * Since: 3.0.0
  */
 void
 ags_buffer_set_buffer_size(AgsBuffer *buffer,
@@ -664,28 +596,24 @@ ags_buffer_set_buffer_size(AgsBuffer *buffer,
   guint old_buffer_size;
   guint word_size;
   
-  pthread_mutex_t *buffer_mutex;
+  GRecMutex *buffer_mutex;
 
   if(!AGS_IS_BUFFER(buffer)){
     return;
   }
       
   /* get buffer mutex */
-  pthread_mutex_lock(ags_buffer_get_class_mutex());
-  
-  buffer_mutex = buffer->obj_mutex;
-  
-  pthread_mutex_unlock(ags_buffer_get_class_mutex());
+  buffer_mutex = AGS_BUFFER_GET_OBJ_MUTEX(buffer);
 
   /* set buffer size */
-  pthread_mutex_lock(buffer_mutex);
+  g_rec_mutex_lock(buffer_mutex);
 
   old_buffer_size = buffer->buffer_size;
   
   buffer->buffer_size = buffer_size;
 
   if(old_buffer_size == buffer->buffer_size){
-    pthread_mutex_unlock(buffer_mutex);    
+    g_rec_mutex_unlock(buffer_mutex);    
 
     return;
   }
@@ -728,7 +656,7 @@ ags_buffer_set_buffer_size(AgsBuffer *buffer,
     }
     break;
   default:
-    pthread_mutex_unlock(buffer_mutex);
+    g_rec_mutex_unlock(buffer_mutex);
     
     g_warning("ags_buffer_set_buffer_size(): unsupported word size");
 
@@ -739,7 +667,7 @@ ags_buffer_set_buffer_size(AgsBuffer *buffer,
     memset(buffer->data + old_buffer_size, 0, (buffer_size - old_buffer_size) * word_size);
   }
 
-  pthread_mutex_unlock(buffer_mutex);
+  g_rec_mutex_unlock(buffer_mutex);
 }
 
 /**
@@ -749,7 +677,7 @@ ags_buffer_set_buffer_size(AgsBuffer *buffer,
  * 
  * Set format.
  * 
- * Since: 2.0.0
+ * Since: 3.0.0
  */
 void
 ags_buffer_set_format(AgsBuffer *buffer,
@@ -759,24 +687,20 @@ ags_buffer_set_format(AgsBuffer *buffer,
 
   guint copy_mode;
 
-  pthread_mutex_t *buffer_mutex;
+  GRecMutex *buffer_mutex;
 
   if(!AGS_IS_BUFFER(buffer)){
     return;
   }
       
   /* get buffer mutex */
-  pthread_mutex_lock(ags_buffer_get_class_mutex());
-  
-  buffer_mutex = buffer->obj_mutex;
-  
-  pthread_mutex_unlock(ags_buffer_get_class_mutex());
+  buffer_mutex = AGS_BUFFER_GET_OBJ_MUTEX(buffer);
 
   /* set format */
-  pthread_mutex_lock(buffer_mutex);
+  g_rec_mutex_lock(buffer_mutex);
 
   if(buffer->format == format){
-    pthread_mutex_unlock(buffer_mutex);
+    g_rec_mutex_unlock(buffer_mutex);
 
     return;
   }
@@ -797,19 +721,19 @@ ags_buffer_set_format(AgsBuffer *buffer,
 
   buffer->format = format;
 
-  pthread_mutex_unlock(buffer_mutex);
+  g_rec_mutex_unlock(buffer_mutex);
 }
 
 /**
  * ags_buffer_sort_func:
  * @a: an #AgsBuffer
- * @b: an #AgsBuffer
+ * @b: an other #AgsBuffer
  * 
  * Sort buffers.
  * 
  * Returns: 0 if equal, -1 if smaller and 1 if bigger offset
  *
- * Since: 2.0.0
+ * Since: 3.0.0
  */
 gint
 ags_buffer_sort_func(gconstpointer a,
@@ -846,9 +770,9 @@ ags_buffer_sort_func(gconstpointer a,
  * 
  * Duplicate a buffer.
  *
- * Returns: the duplicated #AgsBuffer.
+ * Returns: (transfer full): the duplicated #AgsBuffer.
  *
- * Since: 2.0.0
+ * Since: 3.0.0
  */
 AgsBuffer*
 ags_buffer_duplicate(AgsBuffer *buffer)
@@ -857,25 +781,21 @@ ags_buffer_duplicate(AgsBuffer *buffer)
 
   guint copy_mode;
 
-  pthread_mutex_t *buffer_mutex;
+  GRecMutex *buffer_mutex;
 
   if(!AGS_IS_BUFFER(buffer)){
     return(NULL);
   }
   
   /* get buffer mutex */
-  pthread_mutex_lock(ags_buffer_get_class_mutex());
-  
-  buffer_mutex = buffer->obj_mutex;
-  
-  pthread_mutex_unlock(ags_buffer_get_class_mutex());
+  buffer_mutex = AGS_BUFFER_GET_OBJ_MUTEX(buffer);
 
   /* instantiate buffer */  
   buffer_copy = ags_buffer_new();
 
   buffer_copy->flags = 0;
 
-  pthread_mutex_lock(buffer_mutex);
+  g_rec_mutex_lock(buffer_mutex);
 
   buffer_copy->x = buffer->x;
 
@@ -892,7 +812,7 @@ ags_buffer_duplicate(AgsBuffer *buffer)
 					      buffer->data, 1, 0,
 					      buffer_copy->buffer_size, copy_mode);
 
-  pthread_mutex_unlock(buffer_mutex);
+  g_rec_mutex_unlock(buffer_mutex);
 
   return(buffer_copy);
 }
@@ -904,7 +824,7 @@ ags_buffer_duplicate(AgsBuffer *buffer)
  *
  * Returns: the new #AgsBuffer
  *
- * Since: 2.0.0
+ * Since: 3.0.0
  */
 AgsBuffer*
 ags_buffer_new()
