@@ -1,5 +1,5 @@
 /* GSequencer - Advanced GTK Sequencer
- * Copyright (C) 2005-2019 Joël Krähemann
+ * Copyright (C) 2005-2020 Joël Krähemann
  *
  * This file is part of GSequencer.
  *
@@ -80,8 +80,6 @@ enum{
 static gpointer ags_base_plugin_parent_class = NULL;
 static guint base_plugin_signals[LAST_SIGNAL];
 
-static pthread_mutex_t ags_base_plugin_class_mutex = PTHREAD_MUTEX_INITIALIZER;
-
 GType
 ags_base_plugin_get_type (void)
 {
@@ -137,7 +135,7 @@ ags_base_plugin_class_init(AgsBasePluginClass *base_plugin)
    *
    * The assigned AgsUUID
    * 
-   * Since: 2.0.0
+   * Since: 3.0.0
    */
   param_spec = g_param_spec_pointer("uuid",
 				    i18n_pspec("UUID"),
@@ -152,7 +150,7 @@ ags_base_plugin_class_init(AgsBasePluginClass *base_plugin)
    *
    * The assigned filename.
    * 
-   * Since: 2.0.0
+   * Since: 3.0.0
    */
   param_spec = g_param_spec_string("filename",
 				   i18n_pspec("filename of the plugin"),
@@ -168,7 +166,7 @@ ags_base_plugin_class_init(AgsBasePluginClass *base_plugin)
    *
    * The assigned effect.
    * 
-   * Since: 2.0.0
+   * Since: 3.0.0
    */
   param_spec = g_param_spec_string("effect",
 				   i18n_pspec("effect of the plugin"),
@@ -180,11 +178,11 @@ ags_base_plugin_class_init(AgsBasePluginClass *base_plugin)
 				  param_spec);
 
   /**
-   * AgsBasePlugin:plugin-port:
+   * AgsBasePlugin:plugin-port: (type GList(AgsPluginPort)) (transfer full)
    *
    * The assigned #GList-struct containing #AgsPluginPort
    * 
-   * Since: 2.0.0
+   * Since: 3.0.0
    */
   param_spec = g_param_spec_pointer("plugin-port",
 				    i18n_pspec("plugin port of base plugin"),
@@ -199,7 +197,7 @@ ags_base_plugin_class_init(AgsBasePluginClass *base_plugin)
    *
    * The assigned effect-index.
    * 
-   * Since: 2.0.0
+   * Since: 3.0.0
    */
   param_spec = g_param_spec_uint("effect-index",
 				 i18n_pspec("effect index of the plugin"),
@@ -217,7 +215,7 @@ ags_base_plugin_class_init(AgsBasePluginClass *base_plugin)
    *
    * The assigned plugin.so
    * 
-   * Since: 2.0.0
+   * Since: 3.0.0
    */
   param_spec = g_param_spec_pointer("plugin-so",
 				    i18n_pspec("plugin.so"),
@@ -232,7 +230,7 @@ ags_base_plugin_class_init(AgsBasePluginClass *base_plugin)
    *
    * The assigned UI filename.
    * 
-   * Since: 2.0.0
+   * Since: 3.0.0
    */
   param_spec = g_param_spec_string("ui-filename",
 				   i18n_pspec("UI filename of the plugin"),
@@ -248,7 +246,7 @@ ags_base_plugin_class_init(AgsBasePluginClass *base_plugin)
    *
    * The assigned ui-effect.
    * 
-   * Since: 2.0.0
+   * Since: 3.0.0
    */
   param_spec = g_param_spec_string("ui-effect",
 				   i18n_pspec("UI effect of the plugin"),
@@ -264,7 +262,7 @@ ags_base_plugin_class_init(AgsBasePluginClass *base_plugin)
    *
    * The assigned ui-effect-index.
    * 
-   * Since: 2.0.0
+   * Since: 3.0.0
    */
   param_spec = g_param_spec_uint("ui-effect-index",
 				 i18n_pspec("UI effect-index of the plugin"),
@@ -282,7 +280,7 @@ ags_base_plugin_class_init(AgsBasePluginClass *base_plugin)
    *
    * The assigned ui_plugin.so
    * 
-   * Since: 2.0.0
+   * Since: 3.0.0
    */
   param_spec = g_param_spec_pointer("ui-plugin-so",
 				    i18n_pspec("ui_plugin.so"),
@@ -297,7 +295,7 @@ ags_base_plugin_class_init(AgsBasePluginClass *base_plugin)
    *
    * The assigned ui-plugin.
    * 
-   * Since: 2.0.0
+   * Since: 3.0.0
    */
   param_spec = g_param_spec_object("ui-plugin",
 				   i18n_pspec("ui-plugin of the plugin"),
@@ -332,7 +330,7 @@ ags_base_plugin_class_init(AgsBasePluginClass *base_plugin)
    * 
    * Returns: the new plugin instance
    * 
-   * Since: 2.0.0
+   * Since: 3.0.0
    */
   base_plugin_signals[INSTANTIATE] =
     g_signal_new("instantiate",
@@ -356,7 +354,7 @@ ags_base_plugin_class_init(AgsBasePluginClass *base_plugin)
    * 
    * Returns: the new plugin instance
    * 
-   * Since: 2.2.18
+   * Since: 3.0.0
    */
   base_plugin_signals[INSTANTIATE_WITH_PARAMS] =
     g_signal_new("instantiate-with-params",
@@ -379,7 +377,7 @@ ags_base_plugin_class_init(AgsBasePluginClass *base_plugin)
    *
    * The ::connect-port signal creates a new instance of plugin.
    * 
-   * Since: 2.0.0
+   * Since: 3.0.0
    */
   base_plugin_signals[CONNECT_PORT] =
     g_signal_new("connect-port",
@@ -400,7 +398,7 @@ ags_base_plugin_class_init(AgsBasePluginClass *base_plugin)
    *
    * The ::activate signal creates a new instance of plugin.
    * 
-   * Since: 2.0.0
+   * Since: 3.0.0
    */
   base_plugin_signals[ACTIVATE] =
     g_signal_new("activate",
@@ -419,7 +417,7 @@ ags_base_plugin_class_init(AgsBasePluginClass *base_plugin)
    *
    * The ::deactivate signal creates a new instance of plugin.
    * 
-   * Since: 2.0.0
+   * Since: 3.0.0
    */
   base_plugin_signals[DEACTIVATE] =
     g_signal_new("deactivate",
@@ -440,7 +438,7 @@ ags_base_plugin_class_init(AgsBasePluginClass *base_plugin)
    *
    * The ::run signal creates a new instance of plugin.
    * 
-   * Since: 2.0.0
+   * Since: 3.0.0
    */
   base_plugin_signals[RUN] =
     g_signal_new("run",
@@ -455,15 +453,15 @@ ags_base_plugin_class_init(AgsBasePluginClass *base_plugin)
 		 G_TYPE_UINT);
 
   /**
-   * AgsBasePlugin::load_plugin:
+   * AgsBasePlugin::load-plugin:
    * @base_plugin: the plugin to load_plugin
    *
-   * The ::load_plugin signal creates a new instance of plugin.
+   * The ::load-plugin signal creates a new instance of plugin.
    * 
-   * Since: 2.0.0
+   * Since: 3.0.0
    */
   base_plugin_signals[LOAD_PLUGIN] =
-    g_signal_new("load_plugin",
+    g_signal_new("load-plugin",
 		 G_TYPE_FROM_CLASS (base_plugin),
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET (AgsBasePluginClass, load_plugin),
@@ -475,27 +473,10 @@ ags_base_plugin_class_init(AgsBasePluginClass *base_plugin)
 void
 ags_base_plugin_init(AgsBasePlugin *base_plugin)
 {
-  pthread_mutex_t *mutex;
-  pthread_mutexattr_t *attr;
-
   base_plugin->flags = 0;
 
   /* add base plugin mutex */
-  base_plugin->obj_mutexattr = 
-    attr = (pthread_mutexattr_t *) malloc(sizeof(pthread_mutexattr_t));
-  pthread_mutexattr_init(attr);
-  pthread_mutexattr_settype(attr,
-			    PTHREAD_MUTEX_RECURSIVE);
-
-#ifdef __linux__
-  pthread_mutexattr_setprotocol(attr,
-				PTHREAD_PRIO_INHERIT);
-#endif
-
-  base_plugin->obj_mutex = 
-    mutex = (pthread_mutex_t *) malloc(sizeof(pthread_mutex_t));
-  pthread_mutex_init(mutex,
-		     attr);
+  g_rec_mutex_init(&(base_plugin->obj_mutex));
 
   /*  */
   base_plugin->uuid = NULL;
@@ -531,7 +512,7 @@ ags_base_plugin_set_property(GObject *gobject,
 {
   AgsBasePlugin *base_plugin;
 
-  pthread_mutex_t *base_plugin_mutex;
+  GRecMutex *base_plugin_mutex;
 
   base_plugin = AGS_BASE_PLUGIN(gobject);
 
@@ -541,11 +522,11 @@ ags_base_plugin_set_property(GObject *gobject,
   switch(prop_id){
   case PROP_UUID:
     {
-      pthread_mutex_lock(base_plugin_mutex);
+      g_rec_mutex_lock(base_plugin_mutex);
 
       base_plugin->uuid = (AgsUUID *) g_value_get_pointer(value);
 
-      pthread_mutex_unlock(base_plugin_mutex);
+      g_rec_mutex_unlock(base_plugin_mutex);
     }
     break;
   case PROP_FILENAME:
@@ -554,10 +535,10 @@ ags_base_plugin_set_property(GObject *gobject,
 
       filename = (gchar *) g_value_get_string(value);
 
-      pthread_mutex_lock(base_plugin_mutex);
+      g_rec_mutex_lock(base_plugin_mutex);
 
       if(base_plugin->filename == filename){
-	pthread_mutex_unlock(base_plugin_mutex);
+	g_rec_mutex_unlock(base_plugin_mutex);
 	
 	return;
       }
@@ -568,7 +549,7 @@ ags_base_plugin_set_property(GObject *gobject,
 
       base_plugin->filename = g_strdup(filename);
 
-      pthread_mutex_unlock(base_plugin_mutex);
+      g_rec_mutex_unlock(base_plugin_mutex);
     }
     break;
   case PROP_EFFECT:
@@ -577,10 +558,10 @@ ags_base_plugin_set_property(GObject *gobject,
 
       effect = (gchar *) g_value_get_string(value);
 
-      pthread_mutex_lock(base_plugin_mutex);
+      g_rec_mutex_lock(base_plugin_mutex);
 
       if(base_plugin->effect == effect){
-	pthread_mutex_unlock(base_plugin_mutex);
+	g_rec_mutex_unlock(base_plugin_mutex);
 
 	return;
       }
@@ -591,7 +572,7 @@ ags_base_plugin_set_property(GObject *gobject,
 
       base_plugin->effect = g_strdup(effect);
 
-      pthread_mutex_unlock(base_plugin_mutex);
+      g_rec_mutex_unlock(base_plugin_mutex);
     }
     break;
   case PROP_PLUGIN_PORT:
@@ -600,11 +581,11 @@ ags_base_plugin_set_property(GObject *gobject,
 
       plugin_port = (AgsPluginPort *) g_value_get_pointer(value);
 
-      pthread_mutex_lock(base_plugin_mutex);
+      g_rec_mutex_lock(base_plugin_mutex);
 
       if(!AGS_IS_PLUGIN_PORT(plugin_port) ||
 	 g_list_find(base_plugin->plugin_port, plugin_port) != NULL){
-	pthread_mutex_unlock(base_plugin_mutex);
+	g_rec_mutex_unlock(base_plugin_mutex);
 	
 	return;
       }
@@ -613,7 +594,7 @@ ags_base_plugin_set_property(GObject *gobject,
       base_plugin->plugin_port = g_list_prepend(base_plugin->plugin_port,
 						plugin_port);
 
-      pthread_mutex_unlock(base_plugin_mutex);
+      g_rec_mutex_unlock(base_plugin_mutex);
     }
     break;
   case PROP_EFFECT_INDEX:
@@ -622,20 +603,20 @@ ags_base_plugin_set_property(GObject *gobject,
 
       effect_index = g_value_get_uint(value);
 
-      pthread_mutex_lock(base_plugin_mutex);
+      g_rec_mutex_lock(base_plugin_mutex);
 
       base_plugin->effect_index = effect_index;
 
-      pthread_mutex_unlock(base_plugin_mutex);
+      g_rec_mutex_unlock(base_plugin_mutex);
     }
     break;
   case PROP_PLUGIN_SO:
     {
-      pthread_mutex_lock(base_plugin_mutex);
+      g_rec_mutex_lock(base_plugin_mutex);
 
       base_plugin->plugin_so = g_value_get_pointer(value);
 
-      pthread_mutex_unlock(base_plugin_mutex);
+      g_rec_mutex_unlock(base_plugin_mutex);
     }
     break;
   case PROP_UI_FILENAME:
@@ -644,10 +625,10 @@ ags_base_plugin_set_property(GObject *gobject,
 
       ui_filename = (gchar *) g_value_get_string(value);
 
-      pthread_mutex_lock(base_plugin_mutex);
+      g_rec_mutex_lock(base_plugin_mutex);
 
       if(base_plugin->ui_filename == ui_filename){
-	pthread_mutex_unlock(base_plugin_mutex);
+	g_rec_mutex_unlock(base_plugin_mutex);
 
 	return;
       }
@@ -658,7 +639,7 @@ ags_base_plugin_set_property(GObject *gobject,
 
       base_plugin->ui_filename = g_strdup(ui_filename);
 
-      pthread_mutex_unlock(base_plugin_mutex);
+      g_rec_mutex_unlock(base_plugin_mutex);
     }
     break;
   case PROP_UI_EFFECT:
@@ -667,10 +648,10 @@ ags_base_plugin_set_property(GObject *gobject,
 
       ui_effect = (gchar *) g_value_get_string(value);
 
-      pthread_mutex_lock(base_plugin_mutex);
+      g_rec_mutex_lock(base_plugin_mutex);
 
       if(base_plugin->ui_effect == ui_effect){
-	pthread_mutex_unlock(base_plugin_mutex);
+	g_rec_mutex_unlock(base_plugin_mutex);
 
 	return;
       }
@@ -681,7 +662,7 @@ ags_base_plugin_set_property(GObject *gobject,
 
       base_plugin->ui_effect = g_strdup(ui_effect);
 
-      pthread_mutex_unlock(base_plugin_mutex);
+      g_rec_mutex_unlock(base_plugin_mutex);
     }
     break;
   case PROP_UI_EFFECT_INDEX:
@@ -690,20 +671,20 @@ ags_base_plugin_set_property(GObject *gobject,
 
       ui_effect_index = g_value_get_uint(value);
 
-      pthread_mutex_lock(base_plugin_mutex);
+      g_rec_mutex_lock(base_plugin_mutex);
 
       base_plugin->ui_effect_index = ui_effect_index;
 
-      pthread_mutex_unlock(base_plugin_mutex);
+      g_rec_mutex_unlock(base_plugin_mutex);
     }
     break;
   case PROP_UI_PLUGIN_SO:
     {
-      pthread_mutex_lock(base_plugin_mutex);
+      g_rec_mutex_lock(base_plugin_mutex);
 
       base_plugin->ui_plugin_so = g_value_get_pointer(value);
 
-      pthread_mutex_unlock(base_plugin_mutex);
+      g_rec_mutex_unlock(base_plugin_mutex);
     }
     break;
   case PROP_UI_PLUGIN:
@@ -712,10 +693,10 @@ ags_base_plugin_set_property(GObject *gobject,
 
       ui_plugin = g_value_get_object(value);
 
-      pthread_mutex_lock(base_plugin_mutex);
+      g_rec_mutex_lock(base_plugin_mutex);
 
       if(base_plugin->ui_plugin == ui_plugin){
-	pthread_mutex_unlock(base_plugin_mutex);
+	g_rec_mutex_unlock(base_plugin_mutex);
 
 	return;
       }
@@ -730,7 +711,7 @@ ags_base_plugin_set_property(GObject *gobject,
       
       base_plugin->ui_plugin = ui_plugin;
 
-      pthread_mutex_unlock(base_plugin_mutex);
+      g_rec_mutex_unlock(base_plugin_mutex);
     }
     break;
   default:
@@ -747,7 +728,7 @@ ags_base_plugin_get_property(GObject *gobject,
 {
   AgsBasePlugin *base_plugin;
 
-  pthread_mutex_t *base_plugin_mutex;
+  GRecMutex *base_plugin_mutex;
 
   base_plugin = AGS_BASE_PLUGIN(gobject);
 
@@ -757,103 +738,103 @@ ags_base_plugin_get_property(GObject *gobject,
   switch(prop_id){
   case PROP_UUID:
     {
-      pthread_mutex_lock(base_plugin_mutex);
+      g_rec_mutex_lock(base_plugin_mutex);
       
       g_value_set_pointer(value, base_plugin->uuid);
 
-      pthread_mutex_unlock(base_plugin_mutex);
+      g_rec_mutex_unlock(base_plugin_mutex);
     }
     break;
   case PROP_FILENAME:
     {
-      pthread_mutex_lock(base_plugin_mutex);
+      g_rec_mutex_lock(base_plugin_mutex);
 
       g_value_set_string(value, base_plugin->filename);
 
-      pthread_mutex_unlock(base_plugin_mutex);
+      g_rec_mutex_unlock(base_plugin_mutex);
     }
     break;
   case PROP_EFFECT:
     {
-      pthread_mutex_lock(base_plugin_mutex);
+      g_rec_mutex_lock(base_plugin_mutex);
 
       g_value_set_string(value, base_plugin->effect);
 
-      pthread_mutex_unlock(base_plugin_mutex);
+      g_rec_mutex_unlock(base_plugin_mutex);
     }
     break;
   case PROP_PLUGIN_PORT:
     {
-      pthread_mutex_lock(base_plugin_mutex);
+      g_rec_mutex_lock(base_plugin_mutex);
       
       g_value_set_pointer(value, g_list_copy_deep(base_plugin->plugin_port,
 						  (GCopyFunc) g_object_ref,
 						  NULL));
 
-      pthread_mutex_unlock(base_plugin_mutex);
+      g_rec_mutex_unlock(base_plugin_mutex);
     }
     break;
   case PROP_EFFECT_INDEX:
     {
-      pthread_mutex_lock(base_plugin_mutex);
+      g_rec_mutex_lock(base_plugin_mutex);
 
       g_value_set_uint(value, base_plugin->effect_index);
 
-      pthread_mutex_unlock(base_plugin_mutex);
+      g_rec_mutex_unlock(base_plugin_mutex);
     }
     break;
   case PROP_PLUGIN_SO:
     {
-      pthread_mutex_lock(base_plugin_mutex);
+      g_rec_mutex_lock(base_plugin_mutex);
       
       g_value_set_pointer(value, base_plugin->plugin_so);
 
-      pthread_mutex_unlock(base_plugin_mutex);
+      g_rec_mutex_unlock(base_plugin_mutex);
     }
     break;
   case PROP_UI_FILENAME:
     {
-      pthread_mutex_lock(base_plugin_mutex);
+      g_rec_mutex_lock(base_plugin_mutex);
 
       g_value_set_string(value, base_plugin->ui_filename);
 
-      pthread_mutex_unlock(base_plugin_mutex);
+      g_rec_mutex_unlock(base_plugin_mutex);
     }
     break;
   case PROP_UI_EFFECT:
     {
-      pthread_mutex_lock(base_plugin_mutex);
+      g_rec_mutex_lock(base_plugin_mutex);
 
       g_value_set_string(value, base_plugin->ui_effect);
 
-      pthread_mutex_unlock(base_plugin_mutex);
+      g_rec_mutex_unlock(base_plugin_mutex);
     }
     break;
   case PROP_UI_EFFECT_INDEX:
     {
-      pthread_mutex_lock(base_plugin_mutex);
+      g_rec_mutex_lock(base_plugin_mutex);
 
       g_value_set_uint(value, base_plugin->ui_effect_index);
 
-      pthread_mutex_unlock(base_plugin_mutex);
+      g_rec_mutex_unlock(base_plugin_mutex);
     }
     break;
   case PROP_UI_PLUGIN_SO:
     {
-      pthread_mutex_lock(base_plugin_mutex);
+      g_rec_mutex_lock(base_plugin_mutex);
       
       g_value_set_pointer(value, base_plugin->ui_plugin_so);
 
-      pthread_mutex_unlock(base_plugin_mutex);
+      g_rec_mutex_unlock(base_plugin_mutex);
     }
     break;
   case PROP_UI_PLUGIN:
     {
-      pthread_mutex_lock(base_plugin_mutex);
+      g_rec_mutex_lock(base_plugin_mutex);
 
       g_value_set_object(value, base_plugin->ui_plugin);
 
-      pthread_mutex_unlock(base_plugin_mutex);
+      g_rec_mutex_unlock(base_plugin_mutex);
     }
     break;
   default:
@@ -894,13 +875,6 @@ ags_base_plugin_finalize(GObject *gobject)
 
   base_plugin = AGS_BASE_PLUGIN(gobject);
 
-  /* destroy object mutex */
-  pthread_mutex_destroy(base_plugin->obj_mutex);
-  free(base_plugin->obj_mutex);
-
-  pthread_mutexattr_destroy(base_plugin->obj_mutexattr);
-  free(base_plugin->obj_mutexattr);
-
   /* uuid */
   if(base_plugin->uuid != NULL){
     ags_uuid_free(base_plugin->uuid);
@@ -927,21 +901,6 @@ ags_base_plugin_finalize(GObject *gobject)
   G_OBJECT_CLASS(ags_base_plugin_parent_class)->finalize(gobject);
 }
 
-/**
- * ags_base_plugin_get_class_mutex:
- * 
- * Use this function's returned mutex to access mutex fields.
- *
- * Returns: the class mutex
- * 
- * Since: 2.0.0
- */
-pthread_mutex_t*
-ags_base_plugin_get_class_mutex()
-{
-  return(&ags_base_plugin_class_mutex);
-}
-
 /**
  * ags_base_plugin_test_flags:
  * @base_plugin: the #AgsBasePlugin
@@ -951,14 +910,14 @@ ags_base_plugin_get_class_mutex()
  * 
  * Returns: %TRUE if flags are set, else %FALSE
  * 
- * Since: 2.0.0
+ * Since: 3.0.0
  */
 gboolean
 ags_base_plugin_test_flags(AgsBasePlugin *base_plugin, guint flags)
 {
   gboolean retval;
   
-  pthread_mutex_t *base_plugin_mutex;
+  GRecMutex *base_plugin_mutex;
 
   if(!AGS_IS_BASE_PLUGIN(base_plugin)){
     return(FALSE);
@@ -968,11 +927,11 @@ ags_base_plugin_test_flags(AgsBasePlugin *base_plugin, guint flags)
   base_plugin_mutex = AGS_BASE_PLUGIN_GET_OBJ_MUTEX(base_plugin);
 
   /* test flags */
-  pthread_mutex_lock(base_plugin_mutex);
+  g_rec_mutex_lock(base_plugin_mutex);
 
   retval = ((flags & (base_plugin->flags)) != 0) ? TRUE: FALSE;
   
-  pthread_mutex_unlock(base_plugin_mutex);
+  g_rec_mutex_unlock(base_plugin_mutex);
 
   return(retval);
 }
@@ -984,12 +943,12 @@ ags_base_plugin_test_flags(AgsBasePlugin *base_plugin, guint flags)
  *
  * Set flags.
  * 
- * Since: 2.0.0
+ * Since: 3.0.0
  */
 void
 ags_base_plugin_set_flags(AgsBasePlugin *base_plugin, guint flags)
 {
-  pthread_mutex_t *base_plugin_mutex;
+  GRecMutex *base_plugin_mutex;
 
   if(!AGS_IS_BASE_PLUGIN(base_plugin)){
     return;
@@ -999,11 +958,11 @@ ags_base_plugin_set_flags(AgsBasePlugin *base_plugin, guint flags)
   base_plugin_mutex = AGS_BASE_PLUGIN_GET_OBJ_MUTEX(base_plugin);
 
   /* set flags */
-  pthread_mutex_lock(base_plugin_mutex);
+  g_rec_mutex_lock(base_plugin_mutex);
 
   base_plugin->flags |= flags;
   
-  pthread_mutex_unlock(base_plugin_mutex);
+  g_rec_mutex_unlock(base_plugin_mutex);
 }
 
 /**
@@ -1013,12 +972,12 @@ ags_base_plugin_set_flags(AgsBasePlugin *base_plugin, guint flags)
  *
  * Unset flags.
  * 
- * Since: 2.0.0
+ * Since: 3.0.0
  */
 void
 ags_base_plugin_unset_flags(AgsBasePlugin *base_plugin, guint flags)
 {
-  pthread_mutex_t *base_plugin_mutex;
+  GRecMutex *base_plugin_mutex;
 
   if(!AGS_IS_BASE_PLUGIN(base_plugin)){
     return;
@@ -1028,23 +987,23 @@ ags_base_plugin_unset_flags(AgsBasePlugin *base_plugin, guint flags)
   base_plugin_mutex = AGS_BASE_PLUGIN_GET_OBJ_MUTEX(base_plugin);
 
   /* unset flags */
-  pthread_mutex_lock(base_plugin_mutex);
+  g_rec_mutex_lock(base_plugin_mutex);
 
   base_plugin->flags &= (~flags);
   
-  pthread_mutex_unlock(base_plugin_mutex);
+  g_rec_mutex_unlock(base_plugin_mutex);
 }
 
 /**
  * ags_base_plugin_find_filename:
- * @base_plugin: the #GList-struct containing #AgsBasePlugin
+ * @base_plugin: (element-type AgsAudio.BasePlugin): the #GList-struct containing #AgsBasePlugin
  * @filename: the filename as string
  * 
  * Find filename in @base_plugin #GList-struct of #AgsBasePlugin.
  *
- * Returns: the next matching #GList-struct
+ * Returns: (element-type AgsAudio.BasePlugin) (transfer none): the next matching #GList-struct
  * 
- * Since: 2.0.0
+ * Since: 3.0.0
  */
 GList*
 ags_base_plugin_find_filename(GList *base_plugin, gchar *filename)
@@ -1053,7 +1012,7 @@ ags_base_plugin_find_filename(GList *base_plugin, gchar *filename)
   
   gboolean success;
   
-  pthread_mutex_t *base_plugin_mutex;
+  GRecMutex *base_plugin_mutex;
 
   if(filename == NULL){
     return(NULL);
@@ -1066,13 +1025,13 @@ ags_base_plugin_find_filename(GList *base_plugin, gchar *filename)
     base_plugin_mutex = AGS_BASE_PLUGIN_GET_OBJ_MUTEX(current_base_plugin);
 
     /* check filename */
-    pthread_mutex_lock(base_plugin_mutex);
+    g_rec_mutex_lock(base_plugin_mutex);
 
     success = (current_base_plugin->filename != NULL &&
 	       !g_ascii_strcasecmp(current_base_plugin->filename,
 				   filename)) ? TRUE: FALSE;
     
-    pthread_mutex_unlock(base_plugin_mutex);
+    g_rec_mutex_unlock(base_plugin_mutex);
 
     if(success){
       return(base_plugin);
@@ -1086,15 +1045,15 @@ ags_base_plugin_find_filename(GList *base_plugin, gchar *filename)
 
 /**
  * ags_base_plugin_find_effect:
- * @base_plugin: the #GList-struct containing #AgsBasePlugin
+ * @base_plugin: (element-type AgsAudio.BasePlugin): the #GList-struct containing #AgsBasePlugin
  * @filename: the filename as string
  * @effect: the effect as string
  * 
  * Find @filename and @effect in @base_plugin #GList-struct of #AgsBasePlugin.
  *
- * Returns: the next matching #GList-struct
+ * Returns: (element-type AgsAudio.BasePlugin) (transfer none): the next matching #GList-struct
  * 
- * Since: 2.0.0
+ * Since: 3.0.0
  */
 GList*
 ags_base_plugin_find_effect(GList *base_plugin, gchar *filename, gchar *effect)
@@ -1103,7 +1062,7 @@ ags_base_plugin_find_effect(GList *base_plugin, gchar *filename, gchar *effect)
   
   gboolean success;
   
-  pthread_mutex_t *base_plugin_mutex;
+  GRecMutex *base_plugin_mutex;
 
   while(base_plugin != NULL){
     current_base_plugin = AGS_BASE_PLUGIN(base_plugin->data);
@@ -1112,14 +1071,14 @@ ags_base_plugin_find_effect(GList *base_plugin, gchar *filename, gchar *effect)
     base_plugin_mutex = AGS_BASE_PLUGIN_GET_OBJ_MUTEX(current_base_plugin);
 
     /* check filename and effect*/
-    pthread_mutex_lock(base_plugin_mutex);
+    g_rec_mutex_lock(base_plugin_mutex);
 
     success = (!g_ascii_strcasecmp(current_base_plugin->filename,
 				   filename) &&
 	       !g_ascii_strcasecmp(current_base_plugin->effect,
 				   effect)) ? TRUE: FALSE;
     
-    pthread_mutex_unlock(base_plugin_mutex);
+    g_rec_mutex_unlock(base_plugin_mutex);
 
     if(success){
       return(base_plugin);
@@ -1133,15 +1092,15 @@ ags_base_plugin_find_effect(GList *base_plugin, gchar *filename, gchar *effect)
 
 /**
  * ags_base_plugin_find_ui_effect_index:
- * @base_plugin: the #GList-struct containing #AgsBasePlugin
+ * @base_plugin: (element-type AgsAudio.BasePlugin): the #GList-struct containing #AgsBasePlugin
  * @ui_filename: the UI filename as string
  * @ui_effect_index: the UI effect index
  * 
  * Find @ui_filename and @ui_effect_index in @base_plugin #GList-struct of #AgsBasePlugin.
  *
- * Returns: the next matching #GList-struct
+ * Returns: (element-type AgsAudio.BasePlugin) (transfer none): the next matching #GList-struct
  * 
- * Since: 2.0.0
+ * Since: 3.0.0
  */
 GList*
 ags_base_plugin_find_ui_effect_index(GList *base_plugin, gchar *ui_filename, guint ui_effect_index)
@@ -1150,7 +1109,7 @@ ags_base_plugin_find_ui_effect_index(GList *base_plugin, gchar *ui_filename, gui
   
   gboolean success;
   
-  pthread_mutex_t *base_plugin_mutex;
+  GRecMutex *base_plugin_mutex;
 
   while(base_plugin != NULL){
     current_base_plugin = AGS_BASE_PLUGIN(base_plugin->data);
@@ -1159,13 +1118,13 @@ ags_base_plugin_find_ui_effect_index(GList *base_plugin, gchar *ui_filename, gui
     base_plugin_mutex = AGS_BASE_PLUGIN_GET_OBJ_MUTEX(current_base_plugin);
 
     /* check UI filename and effect index */
-    pthread_mutex_lock(base_plugin_mutex);
+    g_rec_mutex_lock(base_plugin_mutex);
 
     success = (!g_ascii_strcasecmp(current_base_plugin->ui_filename,
 				   ui_filename) &&
 	       current_base_plugin->ui_effect_index == ui_effect_index) ? TRUE: FALSE;
     
-    pthread_mutex_unlock(base_plugin_mutex);
+    g_rec_mutex_unlock(base_plugin_mutex);
 
     if(success){
       return(base_plugin);
@@ -1177,72 +1136,83 @@ ags_base_plugin_find_ui_effect_index(GList *base_plugin, gchar *ui_filename, gui
   return(NULL);
 }
 
+/**
+ * ags_base_plugin_sort_func:
+ * @a: a #AgsBasePlugin
+ * @b: an other #AgsBasePlugin
+ *
+ * Sort function.
+ * 
+ * Returns: a value smaller to, equal to or greater to 0 like strcmp
+ *
+ * Since: 3.0.0
+ */
+gint
+ags_base_plugin_sort_func(gpointer a, gpointer b)
+{
+  AgsBasePlugin *a_plugin;
+  AgsBasePlugin *b_plugin;
+    
+  gchar *a_effect;
+  gchar *b_effect;
+
+  gint retval;
+    
+  GRecMutex *a_plugin_mutex;
+  GRecMutex *b_plugin_mutex;
+
+  /* a and b */
+  a_plugin = AGS_BASE_PLUGIN(a);
+  b_plugin = AGS_BASE_PLUGIN(b);
+    
+  /* get base plugin mutex - a and b */
+  a_plugin_mutex = AGS_BASE_PLUGIN_GET_OBJ_MUTEX(a_plugin);
+  b_plugin_mutex = AGS_BASE_PLUGIN_GET_OBJ_MUTEX(b_plugin);
+
+  /* duplicate effect - a */
+  g_rec_mutex_lock(a_plugin_mutex);
+
+  a_effect = g_strdup(a_plugin->effect);
+    
+  g_rec_mutex_unlock(a_plugin_mutex);
+
+  /* duplicate effect - b */
+  g_rec_mutex_lock(b_plugin_mutex);
+
+  b_effect = g_strdup(b_plugin->effect);
+    
+  g_rec_mutex_unlock(b_plugin_mutex);
+
+  if(a_effect == NULL ||
+     b_effect == 0){
+    return(-1);
+  }
+    
+  /* compare and free */
+  retval = strcmp(a_effect,
+		  b_effect);
+
+  g_free(a_effect);
+  g_free(b_effect);
+    
+  return(retval);
+}
+
 /**
  * ags_base_plugin_sort:
- * @base_plugin: the #GList-struct containing #AgsBasePlugin
+ * @base_plugin: (element-type AgsAudio.BasePlugin) (transfer none): the #GList-struct containing #AgsBasePlugin
  *
  * Sort @base_plugin alphabetically.
  * 
- * Returns: the sorted #GList-struct
+ * Returns: (element-type AgsAudio.BasePlugin) (transfer none): the sorted #GList-struct
  * 
- * Since: 2.0.0
+ * Since: 3.0.0
  */
 GList*
 ags_base_plugin_sort(GList *base_plugin)
 {  
   GList *start;
   
-  auto gint ags_base_plugin_sort_compare_function(gpointer a, gpointer b);
-
-  gint ags_base_plugin_sort_compare_function(gpointer a, gpointer b){
-    AgsBasePlugin *a_plugin;
-    AgsBasePlugin *b_plugin;
-    
-    gchar *a_effect;
-    gchar *b_effect;
-
-    gint retval;
-    
-    pthread_mutex_t *a_plugin_mutex;
-    pthread_mutex_t *b_plugin_mutex;
-
-    /* a and b */
-    a_plugin = AGS_BASE_PLUGIN(a);
-    b_plugin = AGS_BASE_PLUGIN(b);
-    
-    /* get base plugin mutex - a and b */
-    a_plugin_mutex = AGS_BASE_PLUGIN_GET_OBJ_MUTEX(a_plugin);
-    b_plugin_mutex = AGS_BASE_PLUGIN_GET_OBJ_MUTEX(b_plugin);
-
-    /* duplicate effect - a */
-    pthread_mutex_lock(a_plugin_mutex);
-
-    a_effect = g_strdup(a_plugin->effect);
-    
-    pthread_mutex_unlock(a_plugin_mutex);
-
-    /* duplicate effect - b */
-    pthread_mutex_lock(b_plugin_mutex);
-
-    b_effect = g_strdup(b_plugin->effect);
-    
-    pthread_mutex_unlock(b_plugin_mutex);
-
-    if(a_effect == NULL ||
-       b_effect == 0){
-      return(-1);
-    }
-    
-    /* compare and free */
-    retval = strcmp(a_effect,
-		    b_effect);
-
-    g_free(a_effect);
-    g_free(b_effect);
-    
-    return(retval);
-  }
-
   if(base_plugin == NULL){
     return(NULL);
   }
@@ -1252,7 +1222,7 @@ ags_base_plugin_sort(GList *base_plugin)
   while(base_plugin != NULL){
     start = g_list_insert_sorted(start,
 				 base_plugin->data,
-				 (GCompareFunc) ags_base_plugin_sort_compare_function);
+				 (GCompareFunc) ags_base_plugin_sort_func);
 
     base_plugin = base_plugin->next;
   }
@@ -1276,7 +1246,7 @@ ags_base_plugin_apply_port_group_by_prefix(AgsBasePlugin *base_plugin)
  *
  * Returns: the new plugin instance handle
  *
- * Since: 2.0.0
+ * Since: 3.0.0
  */
 gpointer
 ags_base_plugin_instantiate(AgsBasePlugin *base_plugin,
@@ -1308,7 +1278,7 @@ ags_base_plugin_instantiate(AgsBasePlugin *base_plugin,
  *
  * Returns: the new plugin instance handle
  *
- * Since: 2.2.18
+ * Since: 3.0.0
  */
 gpointer
 ags_base_plugin_instantiate_with_params(AgsBasePlugin *base_plugin,
@@ -1340,7 +1310,7 @@ ags_base_plugin_instantiate_with_params(AgsBasePlugin *base_plugin,
  *
  * Connect a plugin instance.
  *
- * Since: 2.0.0
+ * Since: 3.0.0
  */
 void
 ags_base_plugin_connect_port(AgsBasePlugin *base_plugin, gpointer plugin_handle, guint port_index, gpointer data_location)
@@ -1360,7 +1330,7 @@ ags_base_plugin_connect_port(AgsBasePlugin *base_plugin, gpointer plugin_handle,
  *
  * Activate a plugin instance
  *
- * Since: 2.0.0
+ * Since: 3.0.0
  */
 void
 ags_base_plugin_activate(AgsBasePlugin *base_plugin, gpointer plugin_handle)
@@ -1380,7 +1350,7 @@ ags_base_plugin_activate(AgsBasePlugin *base_plugin, gpointer plugin_handle)
  *
  * Deactivat a plugin instance
  *
- * Since: 2.0.0
+ * Since: 3.0.0
  */
 void
 ags_base_plugin_deactivate(AgsBasePlugin *base_plugin, gpointer plugin_handle)
@@ -1397,12 +1367,12 @@ ags_base_plugin_deactivate(AgsBasePlugin *base_plugin, gpointer plugin_handle)
  * ags_base_plugin_run:
  * @base_plugin: the #AgsBasePlugin
  * @plugin_handle: the plugin instance handle
- * @seq_event: the alsa sequencer events
+ * @seq_event: (type gpointer) (transfer none): the alsa sequencer events
  * @frame_count: the frame counts
  *
  * Deactivat a plugin instance
  *
- * Since: 2.0.0
+ * Since: 3.0.0
  */
 void
 ags_base_plugin_run(AgsBasePlugin *base_plugin,
@@ -1427,7 +1397,7 @@ ags_base_plugin_run(AgsBasePlugin *base_plugin,
  *
  * Load the plugin
  *
- * Since: 2.0.0
+ * Since: 3.0.0
  */
 void
 ags_base_plugin_load_plugin(AgsBasePlugin *base_plugin)
@@ -1450,7 +1420,7 @@ ags_base_plugin_load_plugin(AgsBasePlugin *base_plugin)
  *
  * Returns: a new #AgsBasePlugin
  *
- * Since: 2.0.0
+ * Since: 3.0.0
  */
 AgsBasePlugin*
 ags_base_plugin_new(gchar *filename, gchar *effect, guint effect_index)
